@@ -1,5 +1,7 @@
 import logging
+import os
 import re
+import sys
 from copy import deepcopy
 from typing import Any, Dict, List, Union
 
@@ -29,6 +31,36 @@ except Exception:
 uploaded_python_packages = []
 
 
+class FutureLen:
+    '''
+    To solve the problem of delayed acquisition for length of output
+    parameters in the debug mode
+    '''
+
+    def __init__(self, par):
+        self.par = par
+
+    def get(self):
+        return len(self.par.value)
+
+
+class FutureRange:
+    def __init__(self, *args):
+        self.args = args
+
+    def get(self):
+        args = []
+        for i in self.args:
+            if isinstance(i, FutureLen):
+                args.append(i.get())
+            elif isinstance(i, (InputParameter, OutputParameter)):
+                args.append(i.value)
+            else:
+                args.append(i)
+        args = tuple(args)
+        return list(range(*args))
+
+
 def argo_range(
         *args,
 ) -> ArgoVar:
@@ -39,12 +71,7 @@ def argo_range(
     Each argument can be Argo parameter
     """
     if config["mode"] == "debug":
-        args = tuple(i.value if isinstance(i, (InputParameter, OutputParameter
-                                               )) else i for i in args)
-        for i in range(len(args)):
-            if isinstance(args[i], (InputParameter, OutputParameter)):
-                args[i] = args[i].value
-        return list(range(*args))
+        return FutureRange(*args)
     start = 0
     step = 1
     if len(args) == 1:
@@ -85,12 +112,13 @@ def argo_sequence(
             with count, can be an Argo parameter
         format: a printf format string to format the value in the sequence
     """
-    if isinstance(count, ArgoVar):
-        count = "{{=%s}}" % count.expr
-    if isinstance(start, ArgoVar):
-        start = "{{=%s}}" % start.expr
-    if isinstance(end, ArgoVar):
-        end = "{{=%s}}" % end.expr
+    if config["mode"] != "debug":
+        if isinstance(count, ArgoVar):
+            count = "{{=%s}}" % count.expr
+        if isinstance(start, ArgoVar):
+            start = "{{=%s}}" % start.expr
+        if isinstance(end, ArgoVar):
+            end = "{{=%s}}" % end.expr
     return V1alpha1Sequence(count=count, start=start, end=end, format=format)
 
 
@@ -104,7 +132,7 @@ def argo_len(
         param: the Argo parameter which is a list
     """
     if config["mode"] == "debug":
-        return len(param.value)
+        return FutureLen(param)
     if isinstance(param, S3Artifact):
         try:
             path_list = catalog_of_artifact(param)
@@ -408,6 +436,28 @@ class Step:
                 }
             )
         elif self.continue_on_success_ratio is not None:
+            if self.with_param is not None:
+                if isinstance(self.with_param, ArgoVar):
+                    total = argo_len(self.with_param)
+                else:
+                    total = len(self.with_param)
+            elif self.with_sequence is not None:
+                if self.with_sequence.count is not None:
+                    count = self.with_sequence.count
+                    if isinstance(count, ArgoVar):
+                        count = count.expr
+                    total = count
+                start = 0
+                if self.with_sequence.start is not None:
+                    start = self.with_sequence.start
+                    if isinstance(start, ArgoVar):
+                        start = start.expr
+                if self.with_sequence.end is not None:
+                    end = self.with_sequence.end
+                    if isinstance(end, ArgoVar):
+                        end = end.expr
+                    total = "%s > %s ? %s + 1 - %s : %s + 1 - %s" \
+                        % (end, start, end, start, start, end)
             self.check_step = self.__class__(
                 name="%s-check-success-ratio" % self.name,
                 template=CheckSuccessRatio(
@@ -415,6 +465,7 @@ class Step:
                     image_pull_policy=self.util_image_pull_policy),
                 parameters={
                     "success": self.outputs.parameters["dflow_success_tag"],
+                    "total": "{{=%s}}" % total,
                     "threshold": self.continue_on_success_ratio
                 }
             )
@@ -527,16 +578,22 @@ class Step:
                         InputParameter()
                     step.with_sequence.start = "{{=%s}}" % \
                         steps.inputs.parameters["dflow_sequence_start"].expr
+                    self.inputs.parameters["dflow_sequence_start"] = \
+                        InputParameter(value=self.with_sequence.start)
                 if self.with_sequence.end is not None:
                     steps.inputs.parameters["dflow_sequence_end"] = \
                         InputParameter()
                     step.with_sequence.end = "{{=%s}}" % \
                         steps.inputs.parameters["dflow_sequence_end"].expr
+                    self.inputs.parameters["dflow_sequence_end"] = \
+                        InputParameter(value=self.with_sequence.end)
                 if self.with_sequence.count is not None:
                     steps.inputs.parameters["dflow_sequence_count"] = \
                         InputParameter()
                     step.with_sequence.count = "{{=%s}}" % \
                         steps.inputs.parameters["dflow_sequence_count"].expr
+                    self.inputs.parameters["dflow_sequence_count"] = \
+                        InputParameter(value=self.with_sequence.count)
                 self.with_sequence = None
 
         if GLOBAL_CONTEXT.in_context:
@@ -564,8 +621,17 @@ class Step:
                     v.type = self.inputs.parameters[k].type
 
                 if self.inputs.parameters[k].save_as_artifact:
+                    if not v.save_as_artifact and v.step is not None:
+                        raise TypeError("%s is big parameter, but %s is not"
+                                        " big parameter" % (
+                                            self.inputs.parameters[k], v))
                     v.save_as_artifact = True
                 if v.save_as_artifact:
+                    if not self.inputs.parameters[k].save_as_artifact and \
+                            v.step is not None:
+                        raise TypeError("%s is big parameter, but %s is not"
+                                        " big parameter" % (
+                                            v, self.inputs.parameters[k]))
                     self.inputs.parameters[k].save_as_artifact = True
 
             if self.inputs.parameters[k].save_as_artifact and isinstance(v, (
@@ -670,6 +736,7 @@ class Step:
 
         import os
         from copy import copy
+
         from .dag import DAG
         from .steps import Steps
 
@@ -703,41 +770,90 @@ class Step:
                 steps.inputs.parameters[name].value = par.value
 
             for name, art in self.inputs.artifacts.items():
+                if not hasattr(art.source, "local_path") and art.optional:
+                    continue
                 steps.inputs.artifacts[name].local_path = art.source.local_path
 
-            steps.run()
+            if "dflow_key" in steps.inputs.parameters and \
+                    steps.inputs.parameters["dflow_key"].value:
+                step_id = steps.inputs.parameters["dflow_key"].value
+                stepdir = os.path.abspath(step_id)
+                if os.path.exists(stepdir):
+                    self.load_output_parameters(stepdir,
+                                                self.outputs.parameters)
+                    self.load_output_artifacts(stepdir,
+                                               self.outputs.artifacts)
+                    with open(os.path.join(stepdir, "phase"), "r") as f:
+                        self.phase = f.read()
+                    return
+                os.makedirs(stepdir)
+            else:
+                while True:
+                    step_id = self.name + "-" + randstr()
+                    stepdir = os.path.abspath(step_id)
+                    if not os.path.exists(stepdir):
+                        os.makedirs(stepdir)
+                        break
+
+            with open(os.path.join(stepdir, "type"), "w") as f:
+                if isinstance(self.template, Steps):
+                    f.write("Steps")
+                elif isinstance(self.template, DAG):
+                    f.write("DAG")
+            with open(os.path.join(stepdir, "phase"), "w") as f:
+                f.write("Running")
+            self.record_input_parameters(stepdir, steps.inputs.parameters)
+            self.record_input_artifacts(stepdir, steps.inputs.artifacts, None,
+                                        True)
+
+            try:
+                steps.run(context.workflow_id)
+            except Exception:
+                self.phase = "Failed"
+                with open(os.path.join(stepdir, "phase"), "w") as f:
+                    f.write("Failed")
+                raise RuntimeError("Step %s failed" % self)
 
             for name, par in self.outputs.parameters.items():
-                if par.value_from_parameter is not None:
-                    par.value = get_var(par.value_from_parameter, steps).value
-                elif par.value_from_expression is not None:
-                    _if = par.value_from_expression._if
+                par1 = self.template.outputs.parameters[name]
+                if par1.value_from_parameter is not None:
+                    par.value = get_var(par1.value_from_parameter, steps).value
+                elif par1.value_from_expression is not None:
+                    _if = par1.value_from_expression._if
                     _if = render_expr(_if, steps)
                     if eval_bool_expr(_if):
-                        _then = par.value_from_expression._then
+                        _then = par1.value_from_expression._then
                         par.value = get_var(_then, steps).value
                     else:
-                        _else = par.value_from_expression._else
+                        _else = par1.value_from_expression._else
                         par.value = get_var(_else, steps).value
 
             for name, art in self.outputs.artifacts.items():
-                if art._from is not None:
-                    art.local_path = get_var(art._from, steps).local_path
-                elif art.from_expression is not None:
-                    _if = art.from_expression._if
+                art1 = self.template.outputs.artifacts[name]
+                if art1._from is not None:
+                    art.local_path = get_var(art1._from, steps).local_path
+                elif art1.from_expression is not None:
+                    _if = art1.from_expression._if
                     _if = render_expr(_if, steps)
                     if eval_bool_expr(_if):
-                        _then = art.from_expression._then
+                        _then = art1.from_expression._then
                         art.local_path = get_var(_then, steps).local_path
                     else:
-                        _else = art.from_expression._else
+                        _else = art1.from_expression._else
                         art.local_path = get_var(_else, steps).local_path
 
+            self.record_output_parameters(stepdir, self.outputs.parameters)
+            self.record_output_artifacts(stepdir, self.outputs.artifacts)
             self.phase = "Succeeded"
+            with open(os.path.join(stepdir, "phase"), "w") as f:
+                f.write("Succeeded")
             return
 
         if self.with_param is not None or self.with_sequence is not None:
-            if isinstance(self.with_param, (InputParameter, OutputParameter)):
+            if isinstance(self.with_param, FutureRange):
+                item_list = self.with_param.get()
+            elif isinstance(self.with_param, (InputParameter,
+                                              OutputParameter)):
                 item_list = self.with_param.value
             elif isinstance(self.with_param, list):
                 item_list = self.with_param
@@ -745,18 +861,27 @@ class Step:
                 start = 0
                 if self.with_sequence.start is not None:
                     start = self.with_sequence.start
+                    if isinstance(start, FutureLen):
+                        start = start.get()
                     if isinstance(start, (InputParameter, OutputParameter)):
                         start = start.value
                 if self.with_sequence.count is not None:
                     count = self.with_sequence.count
+                    if isinstance(count, FutureLen):
+                        count = count.get()
                     if isinstance(count, (InputParameter, OutputParameter)):
                         count = count.value
                     sequence = list(range(start, start + count))
                 if self.with_sequence.end is not None:
                     end = self.with_sequence.end
+                    if isinstance(end, FutureLen):
+                        end = end.get()
                     if isinstance(end, (InputParameter, OutputParameter)):
                         end = end.value
-                    sequence = list(range(start, end + 1))
+                    if end >= start:
+                        sequence = list(range(start, end + 1))
+                    else:
+                        sequence = list(range(start, end - 1, -1))
                 if self.with_sequence.format is not None:
                     item_list = [self.with_sequence.format % i
                                  for i in sequence]
@@ -812,6 +937,8 @@ class Step:
                 self.exec(context, parameters)
             except Exception:
                 self.phase = "Failed"
+                with open(os.path.join(self.stepdir, "phase"), "w") as f:
+                    f.write("Failed")
                 if not self.continue_on_failed:
                     raise RuntimeError("Step %s failed" % self)
 
@@ -824,6 +951,88 @@ class Step:
             traceback.print_exc()
             queue.put((order, None))
 
+    def record_input_parameters(self, stepdir, parameters):
+        os.makedirs(os.path.join(stepdir, "inputs/parameters"), exist_ok=True)
+        for name, par in parameters.items():
+            par_path = os.path.join(stepdir, "inputs/parameters/%s" % name)
+            with open(par_path, "w") as f:
+                f.write(par.value if isinstance(par.value, str)
+                        else jsonpickle.dumps(par.value))
+            if par.type is not None:
+                os.makedirs(os.path.join(
+                    stepdir, "inputs/parameters/.dflow"), exist_ok=True)
+                with open(os.path.join(
+                        stepdir, "inputs/parameters/.dflow/%s" % name),
+                        "w") as f:
+                    f.write(jsonpickle.dumps({"type": str(par.type)}))
+
+    def record_input_artifacts(self, stepdir, artifacts, item,
+                               ignore_nonexist=False):
+        os.makedirs(os.path.join(stepdir, "inputs/artifacts"), exist_ok=True)
+        for name, art in artifacts.items():
+            art_path = os.path.join(stepdir, "inputs/artifacts/%s" % name)
+            if isinstance(art.source, (InputArtifact, OutputArtifact,
+                                       LocalArtifact)):
+                if art.sub_path is not None:
+                    sub_path = art.sub_path
+                    if item is not None:
+                        sub_path = render_item(sub_path, item)
+                    os.symlink(os.path.join(art.source.local_path, sub_path),
+                               art_path)
+                elif isinstance(
+                        art.source,
+                        InputArtifact) and art.optional and not hasattr(
+                            art.source, 'local_path'):
+                    pass
+                else:
+                    os.symlink(art.source.local_path, art_path)
+            elif isinstance(art.source, str):
+                with open(art_path, "w") as f:
+                    f.write(art.source)
+            elif not ignore_nonexist:
+                raise RuntimeError("Not supported: ", art.source)
+
+    def record_output_parameters(self, stepdir, parameters):
+        os.makedirs(os.path.join(stepdir, "outputs/parameters"), exist_ok=True)
+        for name, par in parameters.items():
+            par_path = os.path.join(stepdir,
+                                    "outputs/parameters/%s" % name)
+            if isinstance(par.value, str):
+                value = par.value
+            else:
+                value = jsonpickle.dumps(par.value)
+            with open(par_path, "w") as f:
+                f.write(value)
+            if par.type is not None:
+                os.makedirs(os.path.join(
+                    stepdir, "outputs/parameters/.dflow"), exist_ok=True)
+                with open(os.path.join(
+                        stepdir, "outputs/parameters/.dflow/%s" % name),
+                        "w") as f:
+                    f.write(jsonpickle.dumps({"type": str(par.type)}))
+
+    def record_output_artifacts(self, stepdir, artifacts):
+        os.makedirs(os.path.join(stepdir, "outputs/artifacts"), exist_ok=True)
+        for name, art in artifacts.items():
+            art_path = os.path.join(stepdir, "outputs/artifacts/%s" % name)
+            os.symlink(art.local_path, art_path)
+
+    def load_output_parameters(self, stepdir, parameters):
+        for name, par in parameters.items():
+            par_path = os.path.join(stepdir,
+                                    "outputs/parameters/%s" % name)
+            with open(par_path, "r") as f:
+                if par.type is None or par.type == str:
+                    par.value = f.read()
+                else:
+                    par.value = jsonpickle.loads(f.read())
+
+    def load_output_artifacts(self, stepdir, artifacts):
+        for name, art in artifacts.items():
+            art_path = os.path.join(stepdir,
+                                    "outputs/artifacts/%s" % name)
+            art.local_path = art_path
+
     def exec(self, context, parameters, item=None):
         """
         directory structure:
@@ -852,25 +1061,16 @@ class Step:
             step_id = parameters["dflow_key"].value
             stepdir = os.path.abspath(step_id)
             if os.path.exists(stepdir):
-                # load parameters
-                for name, par in self.outputs.parameters.items():
-                    par_path = os.path.join(stepdir,
-                                            "outputs/parameters/%s" % name)
-                    with open(par_path, "r") as f:
-                        if par.type is None or par.type == str:
-                            par.value = f.read()
-                        else:
-                            par.value = jsonpickle.loads(f.read())
-
-                # load artifacts
-                for name, art in self.outputs.artifacts.items():
-                    art_path = os.path.join(stepdir,
-                                            "outputs/artifacts/%s" % name)
-                    art.local_path = art_path
+                self.load_output_parameters(stepdir,
+                                            self.outputs.parameters)
+                self.load_output_artifacts(stepdir,
+                                           self.outputs.artifacts)
 
                 os.chdir(cwd)
-                self.phase = "Succeeded"
+                with open(os.path.join(stepdir, "phase"), "r") as f:
+                    self.phase = f.read()
                 return
+            os.makedirs(stepdir)
         else:
             while True:
                 step_id = self.name + "-" + randstr()
@@ -879,38 +1079,22 @@ class Step:
                     os.makedirs(stepdir)
                     break
 
+        self.stepdir = stepdir
+        with open(os.path.join(stepdir, "type"), "w") as f:
+            f.write("Pod")
+        with open(os.path.join(stepdir, "phase"), "w") as f:
+            f.write("Running")
         workdir = os.path.join(stepdir, "workdir")
         os.makedirs(workdir, exist_ok=True)
         os.chdir(workdir)
 
-        # render parameters
-        os.makedirs(os.path.join(stepdir, "inputs/parameters"), exist_ok=True)
-        for name, par in parameters.items():
-            par_path = os.path.join(stepdir, "inputs/parameters/%s" % name)
-            with open(par_path, "w") as f:
-                f.write(par.value if isinstance(par.value, str)
-                        else jsonpickle.dumps(par.value))
+        self.record_input_parameters(stepdir, parameters)
 
-        # render artifacts
-        os.makedirs(os.path.join(stepdir, "inputs/artifacts"), exist_ok=True)
+        self.record_input_artifacts(stepdir, self.inputs.artifacts, item)
+
+        # prepare inputs artifacts
         for name, art in self.inputs.artifacts.items():
             art_path = os.path.join(stepdir, "inputs/artifacts/%s" % name)
-            if isinstance(art.source, (InputArtifact, OutputArtifact,
-                                       LocalArtifact)):
-                if art.sub_path is not None:
-                    sub_path = art.sub_path
-                    if item is not None:
-                        sub_path = render_item(sub_path, item)
-                    os.symlink(os.path.join(art.source.local_path, sub_path),
-                               art_path)
-                else:
-                    os.symlink(art.source.local_path, art_path)
-            elif isinstance(art.source, str):
-                with open(art_path, "w") as f:
-                    f.write(art.source)
-            else:
-                raise RuntimeError("Not supported: ", art.source)
-
             path = self.template.inputs.artifacts[name].path
             if hasattr(self.template, "tmp_root"):
                 path = "%s/%s" % (workdir, path)
@@ -919,7 +1103,12 @@ class Step:
             os.makedirs(os.path.dirname(
                 os.path.abspath(path)), exist_ok=True)
             backup(path)
-            os.symlink(art_path, path)
+            if isinstance(
+                    art.source,
+                    InputArtifact) and art.source is None and art.optional:
+                pass
+            else:
+                os.symlink(art_path, path)
 
         # clean output path
         for art in self.outputs.artifacts.values():
@@ -930,31 +1119,38 @@ class Step:
 
         # render variables in the script
         script = self.template.script
-        if hasattr(self.template, "tmp_root"):
+        if hasattr(self.template, "tmp_root") and self.executor is None:
             # do not modify self.template
             template = deepcopy(self.template)
-            template.tmp_root = "%s/%s" % (workdir, template.tmp_root)
+            template.tmp_root = "%s%s" % (workdir, template.tmp_root)
             template.render_script()
-            if self.executor is not None:
-                if hasattr(self.executor, "work_root"):
-                    self.executor.work_root = "."
-                template = self.executor.render(template)
             script = template.script
         script = render_script(script, parameters,
                                context.workflow_id, step_id)
         script_path = os.path.join(stepdir, "script")
         with open(script_path, "w") as f:
             f.write(script)
-        cmd = " ".join(self.template.command) + " " + script_path
-        ret_code = os.system(cmd)
-        if ret_code != 0:
-            raise RuntimeError("Run [%s] failed" % cmd)
 
-        # save parameters
-        os.makedirs(os.path.join(stepdir, "outputs/parameters"), exist_ok=True)
+        import subprocess
+        args = self.template.command + [script_path]
+        p = subprocess.Popen(
+            args=args,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT,
+        )
+        with open("%s/log.txt" % stepdir, "w") as f:
+            line = p.stdout.readline().decode(sys.stdin.encoding)
+            while line:
+                sys.stdout.write(line)
+                f.write(line)
+                line = p.stdout.readline().decode(sys.stdin.encoding)
+        p.wait()
+        ret_code = p.poll()
+        if ret_code != 0:
+            raise RuntimeError("Run %s failed" % args)
+
+        # generate output parameters
         for name, par in self.outputs.parameters.items():
-            par_path = os.path.join(stepdir,
-                                    "outputs/parameters/%s" % name)
             path = par.value_from_path
             if path is not None:
                 if hasattr(self.template, "tmp_root"):
@@ -964,27 +1160,19 @@ class Step:
                         par.value = f.read()
                     else:
                         par.value = jsonpickle.loads(f.read())
-                os.symlink(path, par_path)
             elif hasattr(par, "value"):
                 if isinstance(par.value, str):
                     par.value = render_script(
                         par.value, parameters, context.workflow_id,
                         step_id)
-                    value = par.value
-                else:
-                    value = jsonpickle.dumps(par.value)
-                with open(par_path, "w") as f:
-                    f.write(value)
+        self.record_output_parameters(stepdir, self.outputs.parameters)
 
         # save artifacts
-        os.makedirs(os.path.join(stepdir, "outputs/artifacts"), exist_ok=True)
         for name, art in self.outputs.artifacts.items():
-            art_path = os.path.join(stepdir, "outputs/artifacts/%s" % name)
             path = art.path
             if hasattr(self.template, "tmp_root"):
                 path = "%s/%s" % (workdir, path)
-            os.symlink(path, art_path)
-            art.local_path = art_path
+            art.local_path = path
             for save in self.template.outputs.artifacts[name].save:
                 if isinstance(save, S3Artifact):
                     key = render_script(save.key, parameters,
@@ -994,15 +1182,18 @@ class Step:
 
                     def link(src, dst):
                         try:
-                            os.link(src, dst)
+                            os.symlink(src, dst)
                         except Exception:
                             pass
-                    shutil.copytree(art_path, save_path, copy_function=link,
+                    shutil.copytree(path, save_path, copy_function=link,
                                     dirs_exist_ok=True)
                     art.local_path = save_path
+        self.record_output_artifacts(stepdir, self.outputs.artifacts)
 
         os.chdir(cwd)
         self.phase = "Succeeded"
+        with open(os.path.join(stepdir, "phase"), "w") as f:
+            f.write("Succeeded")
 
     def exec_with_queue(self, context, parameters, order, queue, item=None):
         try:
@@ -1087,20 +1278,25 @@ def get_var(expr, context):
 def eval_bool_expr(expr):
     # For the original evaluator in argo, please refer to
     # https://github.com/antonmedv/expr
-    import os
-    expr = expr.replace("<=", "-le")
-    expr = expr.replace(">=", "-ge")
-    expr = expr.replace("<", "-lt")
-    expr = expr.replace(">", "-gt")
-    result = os.popen(
-        "sh -c 'if [[ %s ]]; then echo 1; else echo 0; fi'"
-        % expr).read().strip()
-    if result == "1":
-        return True
-    elif result == "0":
-        return False
-    else:
-        raise RuntimeError("Evaluate expression failed: %s" % expr)
+    expr_list = expr.split()
+    operator = expr_list[1]
+
+    assert (len(expr_list) == 3)
+    if operator == "==":
+        return expr_list[0] == expr_list[2]
+    elif operator == "!=":
+        return expr_list[0] != expr_list[2]
+    expr_right = float(expr_list[2])
+    expr_left = float(expr_list[0])
+    if operator == "<=":
+        return expr_left <= expr_right
+    elif operator == "<":
+        return expr_left < expr_right
+    elif operator == ">=":
+        return expr_left >= expr_right
+    elif operator == ">":
+        return expr_left > expr_right
+    raise RuntimeError("Evaluate expression failed: %s" % expr)
 
 
 def render_script(script, parameters, workflow_id=None, step_id=None):
