@@ -1,6 +1,9 @@
 import contextlib
+import hashlib
+import inspect
 import logging
 import os
+import pkgutil
 import random
 import shutil
 import string
@@ -9,6 +12,7 @@ import sys
 import tarfile
 import tempfile
 import uuid
+from functools import partial
 from pathlib import Path
 from typing import List, Optional, Set, Tuple, Union
 
@@ -33,6 +37,23 @@ s3_config = {
 }
 
 
+def set_s3_config(
+    **kwargs,
+) -> None:
+    """
+    Set S3 configurations
+
+    Args:
+        endpoint: endpoint for S3 storage
+        console: console address for S3 storage
+        access_key: access key for S3 storage
+        secret_key: secret key for S3 storage
+        secure: secure or not
+        bucket_name: name of S3 bucket
+    """
+    s3_config.update(kwargs)
+
+
 def download_artifact(
         artifact,
         extract: bool = True,
@@ -46,22 +67,27 @@ def download_artifact(
     Args:
         artifact: artifact to be downloaded
         extract: extract files if the artifact is compressed
+        sub_path: download a subdir of an artifact
+        slice: download a slice of an artifact
         path: local path
         endpoint: endpoint for Minio
         access_key: access key for Minio
         secret_key: secret key for Minio
         secure: secure or not for Minio
         bucket_name: bucket name for Minio
+        skip_exists: skip files with the same MD5
     """
     if config["mode"] == "debug":
-        return assemble_path_list(artifact.local_path)
+        path = kwargs["path"] if "path" in kwargs else "."
+        linktree(artifact.local_path, path)
+        return assemble_path_list(path, remove=True)
 
     if hasattr(artifact, "s3"):
         key = artifact.s3.key
     elif hasattr(artifact, "key"):
         key = artifact.key
     else:
-        raise NotImplementedError()
+        raise FileNotFoundError("The artifact does not exist in the storage.")
 
     if slice is not None:
         sub_path = path_list_of_artifact(artifact)[slice]
@@ -151,6 +177,8 @@ def upload_artifact(
         if config["mode"] == "debug":
             os.makedirs("upload", exist_ok=True)
             resdir = shutil.move(tmpdir, "upload")
+            # To prevent exception in destruction
+            os.makedirs(tmpdir, exist_ok=True)
             return LocalArtifact(local_path=os.path.abspath(resdir))
 
         if archive == "tar":
@@ -214,6 +242,14 @@ def copy_artifact(src, dst, sort=False) -> S3Artifact:
     return S3Artifact(key=dst_key)
 
 
+def get_md5(f):
+    md5 = hashlib.md5()
+    with open(f, "rb") as fd:
+        for chunk in iter(lambda: fd.read(4096), b""):
+            md5.update(chunk)
+    return md5.hexdigest()
+
+
 def download_s3(
         key: str,
         path: os.PathLike = None,
@@ -223,6 +259,7 @@ def download_s3(
         secret_key: str = None,
         secure: bool = None,
         bucket_name: str = None,
+        skip_exists: bool = False,
         **kwargs,
 ) -> str:
     if endpoint is None:
@@ -249,6 +286,15 @@ def download_s3(
                 file_path = os.path.join(path, os.path.basename(key))
             else:
                 file_path = os.path.join(path, rel_path)
+
+            if skip_exists and os.path.isfile(file_path):
+                remote_md5 = client.stat_object(
+                    bucket_name=bucket_name, object_name=obj.object_name).etag
+                local_md5 = get_md5(file_path)
+                if remote_md5 == local_md5:
+                    logging.debug("skip object: %s" % obj.object_name)
+                    continue
+
             client.fget_object(bucket_name=bucket_name,
                                object_name=obj.object_name,
                                file_path=file_path)
@@ -446,8 +492,8 @@ def assemble_path_list(art_path, remove=False):
                     for item in jsonpickle.loads(fd.read())['path_list']:
                         if item not in dflow_list:
                             dflow_list.append(item)  # remove duplicate
-                if remove:
-                    os.remove(os.path.join(catalog_dir, f))
+            if remove:
+                shutil.rmtree(catalog_dir)
         if len(dflow_list) > 0:
             path_list = list(map(lambda x: os.path.join(
                 art_path, x) if x is not None else None,
@@ -557,3 +603,38 @@ def run_command(
     if raise_error:
         assert return_code == 0, "Command %s failed: \n%s" % (cmd, err)
     return return_code, out, err
+
+
+def find_subclass(module, cls):
+    ops = []
+    for _, m in inspect.getmembers(module):
+        if inspect.isclass(m) and issubclass(m, cls) and m != cls and \
+                m not in ops:
+            ops.append(m)
+    if hasattr(module, "__path__"):
+        for path in module.__path__:
+            for dir, _, _ in os.walk(path):
+                pkg = (module.__name__ + dir[len(path):]).replace("/", ".")
+                for _, name, _ in pkgutil.iter_modules([dir]):
+                    try:
+                        mod = __import__(pkg + "." + name, fromlist=pkg)
+                        for _, m in inspect.getmembers(mod):
+                            if inspect.isclass(m) and issubclass(m, cls) and \
+                                    m != cls and m not in ops:
+                                ops.append(m)
+                    except Exception:
+                        pass
+    return ops
+
+
+def linktree(src, dst, func=os.symlink):
+    shutil.copytree(src, dst, copy_function=partial(link, func=func),
+                    dirs_exist_ok=True)
+
+
+def link(src, dst, func=os.symlink):
+    if os.path.exists(dst):
+        if os.path.samefile(src, dst):
+            return
+        os.remove(dst)
+    func(src, dst)
